const hasOwnProperty = Object.prototype.hasOwnProperty;
function preserve(obj) {
    const result = Object.create(null);
    Object.keys(obj).forEach(key => {
        result[key] = obj[key];
    });
    Object.getOwnPropertySymbols(obj).forEach(key => {
        result[key] = obj[key];
    });
    return Object.freeze(result);
}
export function createMatcher(kindKey, kindDict) {
    const kinds = preserve(kindDict);
    function match(matched, options, defaultTo) {
        const kind = matched[kindKey];
        if (hasOwnProperty.call(options, kind)) {
            const handler = options[kind];
            return handler(kinds[kind](matched));
        }
        if (defaultTo) {
            return defaultTo();
        }
        throw new TypeError("unknown kind");
    }
    return match;
}
